function im_d = denoiseFullSFpyr(im,noiseVar,noise,opts);

% function im_d = denoiseFullSFpyr(im,noiseVar,noise,opts);
%
% Denoise image using a seperable steerable Wavelet pyramid with 
% oriented highpass residual bands. Called from 'denoise_main' 
%
% where
%   'im'        is the noisy image
%   'noiseVar'  is the noise variance
%   'noise'     a noisy image to estimate the covariance
%   'opts'      a set of options, see 'newOptions'
%
% Peter-Vincent Gehler 15.March 2005

verbose = 1;

% Create the pyramid representation of the image and the noise ...
if verbose, fprintf('building Wavelet pyramids ...'); end
[pyr,pind] = buildFullSFpyr2(im,opts.nLevels,opts.nOrientations-1);
[pyrN,pind] = buildFullSFpyr2(noise,opts.nLevels,opts.nOrientations-1);
if verbose ,fprintf('done\n'); end

% ... for the new denoised pyramid ...
pyrnew = real(pyr);

numOfBands = size(pind,1);

% The first band is a virtual high pass residual and only included for
% compability reasons. 
if verbose,fprintf('1st band does not exist\n');end

% ... for each subband ...
for nband = 2:numOfBands-1
    if verbose,fprintf('denoising band no %d\n',nband);end
    % ... decide if to include the parent ...
    prnt = opts.parent & (nband+opts.nOrientations < numOfBands-1);
    % ... denoise ...
    newBand = denoise_band(pyr,pyrN,pind,nband,opts,prnt);
    % ... and store the result.
    pyrnew(pyrBandIndices(pind,nband)) = newBand(:);
end

clear global gl_avg2
clear global gl_avg4

% ... reconstruct the image and return. 
if verbose,fprintf('inverting new wavelet pyramid ...');end
im_d = reconFullSFpyr2(pyrnew,pind);
if verbose,fprintf('done\n');end
