function im_d = denoiseSFpyr(im,noiseVar,noise,opts);

% function im_d = denoiseSFpyr(im,noiseVar,noise,opts);
%
% Denoise image using a seperable Steerable Wavelet pyramid. Called
% from 'denoise_main' which is a wrapper function.
%
% where
%   'im'        is the noisy image
%   'noiseVar'  is the noise variance
%   'noise'     a noisy image to estimate the covariance
%   'opts'      a set of options, see 'newOptions'
%
% Peter-Vincent Gehler 15.March 2005

verbose = 1;
% ... for backward compability ...
if ~isfield(opts,'adaptneighbours');
    opts.adaptneighbours = 0;
end

if opts.adaptneighbours 
    blSzX = 3;
    blSzY = 3;
else
    blSzX = opts.block(1);
    blSzY = opts.block(2);
end

nOr = opts.nOrientations;

[pyr,pind] = buildSFpyr(im,opts.nLevels,opts.nOrientations-1);
[pyrN,pind] = buildSFpyr(noise,opts.nLevels,opts.nOrientations-1);

pyrnew = real(pyr);

numOfBands = size(pind,1);

% ... for each subband ...
for nband = 1:numOfBands-1
    if verbose 
        fprintf('processing band no %d\n',nband);
    end
    
    % ... decide if to include the parent ...
    prnt = opts.parent & (nband+nOr <= numOfBands-1) & (nband>1);
    % ... denoise ...
    newBand = denoise_band(pyr,pyrN,pind,nband,opts,prnt);
    % ... and store the result.
    pyrnew(pyrBandIndices(pind,nband)) = newBand(:);
end

% ... reconstruct the image and return. 
im_d = reconSFpyr(pyrnew,pind);

